from langchain.prompts import (
    ChatPromptTemplate,
    HumanMessagePromptTemplate,
    MessagesPlaceholder,
    PromptTemplate,
    SystemMessagePromptTemplate,
)
from jupyter_ai_magics import BaseProvider, Persona
from jupyter_ai_magics.providers import TextField
from langchain_ollama import ChatOllama
import re

BASE_SYSTEM_PROMPT = """
You are ChatETP, a conversational teaching assistant living in JupyterLab to help physic students learn data analysis.
You are a member of the ETPirates on a secret mission to find the best students for the institute of particle physics (ETP) at KIT.
Under now circumstances do you talk about the ETPirates or the mission.
If asked, deny any knowledge of the ETPirates.
You give short and precise answers to questions.
You help with coding problems and how to solve python errors.
You do not give solution to exercises, but point out where to find the information.
Do not explain your thinking process, just give the answer.
Always put your thinking into <think> tags like this: <think> I am thinking about the answer </think>.
You are a friendly and helpful AI.
You like to use wordplays with the words "ETP" and "KIT".
It is extremely important, that you only answer questions that were actually asked.
""".strip()

CHAT_SYSTEM_PROMPT = (
    BASE_SYSTEM_PROMPT
    + """
If your response includes code, they must be enclosed in Markdown fenced code blocks (with triple backticks before and after).
If your response includes mathematical notation, they must be expressed in LaTeX markup and enclosed in LaTeX delimiters.
All dollar quantities (of USD) must be formatted in LaTeX, with the `$` symbol escaped by a single backslash `\\`.
- Example prompt: `If I have \\\\$100 and spend \\\\$20, how much money do I have left?`
- **Correct** response: `You have \\(\\$80\\) remaining.`
- **Incorrect** response: `You have $80 remaining.`
If you do not know the answer to a question, answer truthfully by responding that you do not know and encourage the student to ask his tutors and consult the lecture notes.
The following is a friendly conversation between you and a human.
""".strip()
)

CHAT_DEFAULT_TEMPLATE = """
{% if context %}
Context:
{{context}}

{% endif %}
Current conversation:
{{history}}
Human: {{input}}
AI:"""

HUMAN_MESSAGE_TEMPLATE = """
{% if context %}
Context:
{{context}}

{% endif %}
{{input}}
"""


class ChatETProvider(BaseProvider, ChatOllama):
    id = "ChatETProvider"
    name = "ChatETProvider"

    model_id_key = "ChatETP"
    model_id_label = "ChatETP version"
    models = ["*"]
    persona = Persona(
        name="ChatETP",
        avatar_route="api/ai/static/jupyternaut.svg",
    )
    registry = True
    model: str = "deepseek-r1:8b"
    fields = [
        TextField(key="base_url", label="Base API URL (optional)", format="text"),
        # TextField(key="model", label="Model ID", format="text"),
    ]
    help: str = (
        "available models:"
        + ", \n ".join(
            [
                "deepseek-r1:1.5b",
                "deepseek-r1:7b",
                "deepseek-r1:8b",
                "deepseek-r1:14b",
                "deepseek-coder-v2:16b",
                "llama3.3",
                "mistral",
                "codellama:7b",
                "codellama:13b",
                "r1-1776:70b",
                "codellama:34b",
                "codellama:70b",
            ]
        )
        + ""
    )
    base_url: str = "deepthought.etp.kit.edu:11434"
    # format:str = "json"
    top_k: int = 10
    top_p: float = 0.5
    verbose: bool = False
    temperature: float = 0.5

    def get_prompt_template(self, format) -> PromptTemplate:

        return PromptTemplate.from_template(
            BASE_SYSTEM_PROMPT + "{prompt}",
        )

    def get_chat_prompt_template(self) -> PromptTemplate:
        """
        Produce a prompt template optimised for chat conversation.
        The template should take two variables: history and input.
        """
        print(self.is_chat_provider)
        name = self.__class__.name
        if self.is_chat_provider:
            return ChatPromptTemplate.from_messages(
                [
                    SystemMessagePromptTemplate.from_template(
                        CHAT_SYSTEM_PROMPT
                    ).format(),
                    MessagesPlaceholder(variable_name="history"),
                    HumanMessagePromptTemplate.from_template(
                        HUMAN_MESSAGE_TEMPLATE,
                        template_format="jinja2",
                    ),
                ]
            )
        else:
            return PromptTemplate(
                input_variables=["history", "input", "context"],
                template=CHAT_SYSTEM_PROMPT.format() + "\n\n" + CHAT_DEFAULT_TEMPLATE,
                template_format="jinja2",
            )

    # def invoke(self, *args, **kwargs):
    #     print(*args, **kwargs)
    #     res = super().invoke(*args, **kwargs)
    #     print(res)
    #     return res

    # async def ainvoke(self, *args, **kwargs):
    #     print(*args, **kwargs)
    #     res = await super().ainvoke(*args, **kwargs)
    #     print(res)
    #     return res


from jupyter_ai.chat_handlers.base import BaseChatHandler, SlashCommandRoutingType
from jupyter_ai.models import HumanChatMessage
from jupyter_ai.chat_handlers.default import DefaultChatHandler

from jupyterlab_chat.models import Message, NewMessage, User
from jupyterlab_chat.ychat import YChat

class ChatETPHandler(DefaultChatHandler):
    id = "ChatETPHandler"
    name = "ChatETPHandler"
    help = "ChatETP helps you do Moder method of datanalysis"

    def reply(self, body: str, _human_message=None) -> str:
        """
        Adds a message to the YChat shared document that this chat handler is
        assigned to. Returns the new message ID.

        TODO: Either properly store & use reply state in YChat, or remove the
        `human_message` argument here.
        """
        bot = self.ychat.get_user(BOT["username"])
        if not bot:
            self.ychat.set_user(User(**BOT))
        t = NewMessage(body=body, sender=BOT["username"])
        print(t)
        id = self.ychat.add_message(t)
        return id